{% load l10n cascade_tags %}{% spaceless %}

<div uib-carousel interval="{{ instance.options.interval|default:5000|unlocalize }}" active="0"{# style="{{ instance.inline_styles }}" #}>
	{% for slide in instance.child_plugin_instances %}
	<div uib-slide index="{{ forloop.counter0 }}">{% render_plugin slide %}</div>
	{% endfor %}
</div>
{% endspaceless %}