{% extends "cascade/admin/change_form.html" %}
{% load i18n cms_tags %}

{% block extrastyle %}
	{{ block.super }}
	{% if icon_font %}
<link rel="stylesheet" type="text/css" href="{{ icon_font.get_stylesheet_url }}" />
<script type="application/json" id="cascade_iconfont_config_data">
{{ icon_font.config_data_as_json|safe }}
</script>
	{% endif %}
{% endblock %}

{% block field_sets %}
	{% if icon_font or not require_icon_font %}
		{{ block.super }}
	{% else %}
<div class="module">
	<h2>{% trans "Missing Icon Font for this CMS page" %}</h2>
		{% if request.META.HTTP_REFERER %}
	<p>{% blocktrans %}
	In the toolbar, navigate to <strong>Page &gt; Choose Icon Font</strong> and select a font.
	{% endblocktrans %}</p>
		{% else %}
	<p>{% blocktrans %}
	Check that your browser sends an HTTP-Referer, otherwise this editor can not find out which Icon Font has been selected for the current page.
	{% endblocktrans %}</p>
		{% endif %}
</div>
	{% endif %}
{% endblock %}
